import java.util.*;

public abstract class Humain implements Comparable<Humain> {

    protected static Random loto = new Random(Calendar.getInstance().getTimeInMillis());
    protected int age;
    protected int poids;
    protected String nom;
    protected int esperanceVie;

    Humain(String nom) {
        this.nom = nom;
        age = 0;
        poids = 3;
        setEsperanceVie();
    }

    Humain(int age, int poids, String nom) {
        this.age = age;
        this.poids = poids;
        this.nom = nom;
        setEsperanceVie();
    }

    void setNom(String nom) {
        this.nom = nom;
    }

    void setAge(int age) {
        if (age < 0) age = 0;
        this.age = age;
    }

    void setPoids(int poids) {
        if (poids < 3) poids = 3;
        this.poids = poids;
    }

    int getAge() {
        return age;
    }

    int getPoids() {
        return poids;
    }

    String getNom() {
        return nom;
    }

    // le code dépend du sexe, donc abstract
    protected abstract void setEsperanceVie();

    public void vieillir(Aging aging) {
        age++;
        // on applique les modifs de poids et de fertilité dues à l'âge.
        aging.apply(this);
    }

    public void grossir(int p) {
        poids += p;
    }

    public boolean isDead() {
        if (age > esperanceVie) {
            return true;
        }
        if ((age > 20) && (poids < 20)) return true;

        return false;
    }

    public void print() {
        System.out.println(nom+" : "+age+" ans, "+poids+" kg, mourra à "+esperanceVie+" ans");
    }

    public abstract boolean isHomme();
    public abstract boolean isFemme();
    public abstract boolean isGarcon();
    public abstract boolean isFille();

    // le code dépend du sexe, donc abstract
    public abstract Humain rencontre(Humain h, boolean doGrowth,  Breeding breeding);

    // la comparaison pour trier en fonction de l'âge
    public int compareTo(Humain h) {
        if (age == h.age) {
            return 0;
        }
        else if (age < h.age) {
            return -1;
        }
        else {
            return 1;
        }
    }
}
